@@ -207,7 +207,7 @@ package Sidef::Sys::Sys {
 
     sub refaddr {
         my ($self, $obj) = @_;
-        Sidef::Types::Number::Number->new(Scalar::Util::refaddr($obj));
+        Sidef::Types::Number::Number->_set_uint(Scalar::Util::refaddr($obj));
     }
 
     sub reftype {
@@ -237,7 +237,7 @@ package Sidef::Sys::Sys {
 
     sub system {
         my ($self, @args) = @_;
-        Sidef::Types::Number::Number->new(scalar CORE::system(@args));
+        Sidef::Types::Number::Number->_set_int(scalar CORE::system(@args));
     }
 
     *run = \&system;
